/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_CAMPRKPERSERVICE_CAMERAPARKINGPERCEPTIONSERVICE_SKELETON_H
#define JAC_AP_CAMPRKPERSERVICE_CAMERAPARKINGPERCEPTIONSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/camprkperservice/cameraparkingperceptionservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace camprkperservice {
namespace skeleton {
namespace events
{
    using CamPrk2CamPrkPerceptionEvent = ara::com::internal::skeleton::event::EventAdapter<::perception::Cam2CamPerceptionStruct>;
    static constexpr ara::com::internal::EntityId CameraParkingPerceptionServiceCamPrk2CamPrkPerceptionEventId = 41953U; //CamPrk2CamPrkPerceptionEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class CameraParkingPerceptionServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultCamPrk2CamPrkPerceptionEvent = CamPrk2CamPrkPerceptionEvent.Initialize();
        ThrowError(resultCamPrk2CamPrkPerceptionEvent);
    }

    CameraParkingPerceptionServiceSkeleton& operator=(const CameraParkingPerceptionServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceId, mode)),
              CamPrk2CamPrkPerceptionEvent(ptr->GetSkeleton(), events::CameraParkingPerceptionServiceCamPrk2CamPrkPerceptionEventId, ::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceSpec, mode)),
              CamPrk2CamPrkPerceptionEvent(ptr->GetSkeleton(), events::CameraParkingPerceptionServiceCamPrk2CamPrkPerceptionEventId, ::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceContainer, mode)),
              CamPrk2CamPrkPerceptionEvent(ptr->GetSkeleton(), events::CameraParkingPerceptionServiceCamPrk2CamPrkPerceptionEventId, ::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              CamPrk2CamPrkPerceptionEvent(std::move(other.CamPrk2CamPrkPerceptionEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                CamPrk2CamPrkPerceptionEvent = std::move(other.CamPrk2CamPrkPerceptionEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::CamPrk2CamPrkPerceptionEvent GetCamPrk2CamPrkPerceptionEvent()
        {
            return std::move(CamPrk2CamPrkPerceptionEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = CamPrk2CamPrkPerceptionEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::CamPrk2CamPrkPerceptionEvent CamPrk2CamPrkPerceptionEvent;
    };
    explicit CameraParkingPerceptionServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceId, mode)),
          CamPrk2CamPrkPerceptionEvent(skeletonAdapter->GetSkeleton(), events::CameraParkingPerceptionServiceCamPrk2CamPrkPerceptionEventId, ::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit CameraParkingPerceptionServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceSpec, mode)),
          CamPrk2CamPrkPerceptionEvent(skeletonAdapter->GetSkeleton(), events::CameraParkingPerceptionServiceCamPrk2CamPrkPerceptionEventId, ::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit CameraParkingPerceptionServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceContainer, mode)),
          CamPrk2CamPrkPerceptionEvent(skeletonAdapter->GetSkeleton(), events::CameraParkingPerceptionServiceCamPrk2CamPrkPerceptionEventId, ::jac::ap::camprkperservice::CameraParkingPerceptionService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    CameraParkingPerceptionServiceSkeleton(const CameraParkingPerceptionServiceSkeleton&) = delete;

    CameraParkingPerceptionServiceSkeleton(CameraParkingPerceptionServiceSkeleton&&) = default;
    CameraParkingPerceptionServiceSkeleton& operator=(CameraParkingPerceptionServiceSkeleton&&) = default;
    CameraParkingPerceptionServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          CamPrk2CamPrkPerceptionEvent(token.GetCamPrk2CamPrkPerceptionEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~CameraParkingPerceptionServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&CameraParkingPerceptionServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::CamPrk2CamPrkPerceptionEvent CamPrk2CamPrkPerceptionEvent;
};
} // namespace skeleton
} // namespace camprkperservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_CAMPRKPERSERVICE_CAMERAPARKINGPERCEPTIONSERVICE_SKELETON_H
